@@ -1,41 +1,21 @@
-
 /*
- * $Id$
- *
- * DEBUG: section 81    aio_xxx() POSIX emulation on Windows
- * AUTHOR: Guido Serassio <serassio@squid-cache.org>
- *
- * SQUID Web Proxy Cache          http://www.squid-cache.org/
- * ----------------------------------------------------------
- *
- *  Squid is the result of efforts by numerous individuals from
- *  the Internet community; see the CONTRIBUTORS file for full
- *  details.   Many organizations have provided support for Squid's
- *  development; see the SPONSORS file for full details.  Squid is
- *  Copyrighted (C) 2001 by the Regents of the University of
- *  California; see the COPYRIGHT file for full details.  Squid
- *  incorporates software developed and/or copyrighted by other
- *  sources; see the CREDITS file for full details.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
-#include "squid-old.h"
+/* DEBUG: section 81    aio_xxx() POSIX emulation on Windows */
+
+#include "squid.h"
 #include "comm.h"
-#include "aio_win32.h"
+#include "DiskIO/AIO/aio_win32.h"
+#include "fd.h"
+#include "StatCounters.h"
+#include "win32.h"
+
+#include <cerrno>
 
 #if _SQUID_WINDOWS_
 VOID CALLBACK IoCompletionRoutine(DWORD dwErrorCode,
@@ -50,7 +30,6 @@ VOID CALLBACK IoCompletionRoutine(DWORD dwErrorCode,
     xfree(lpOverlapped);
 }
 
-
 int aio_read(struct aiocb *aiocbp)
 {
     LPOVERLAPPED Overlapped;
@@ -100,7 +79,7 @@ int aio_read(struct aiocb *aiocbp)
     /* Test to see if the I/O was queued successfully. */
     if (!IoOperationStatus) {
         errno = GetLastError();
-        debugs(81,1, "aio_read: GetLastError=" << errno  );
+        debugs(81, DBG_IMPORTANT, "aio_read: GetLastError=" << errno  );
         return -1;
     }
 
@@ -110,7 +89,6 @@ int aio_read(struct aiocb *aiocbp)
     return 0;
 }
 
-
 int aio_read64(struct aiocb64 *aiocbp)
 {
     LPOVERLAPPED Overlapped;
@@ -152,7 +130,7 @@ int aio_read64(struct aiocb64 *aiocbp)
     /* Test to see if the I/O was queued successfully. */
     if (!IoOperationStatus) {
         errno = GetLastError();
-        debugs(81, 1, "aio_read: GetLastError=" << errno  );
+        debugs(81, DBG_IMPORTANT, "aio_read: GetLastError=" << errno  );
         return -1;
     }
 
@@ -162,7 +140,6 @@ int aio_read64(struct aiocb64 *aiocbp)
     return 0;
 }
 
-
 int aio_write(struct aiocb *aiocbp)
 {
     LPOVERLAPPED Overlapped;
@@ -212,7 +189,7 @@ int aio_write(struct aiocb *aiocbp)
     /* Test to see if the I/O was queued successfully. */
     if (!IoOperationStatus) {
         errno = GetLastError();
-        debugs(81, 1, "aio_write: GetLastError=" << errno  );
+        debugs(81, DBG_IMPORTANT, "aio_write: GetLastError=" << errno  );
         return -1;
     }
 
@@ -222,7 +199,6 @@ int aio_write(struct aiocb *aiocbp)
     return 0;
 }
 
-
 int aio_write64(struct aiocb64 *aiocbp)
 {
     LPOVERLAPPED Overlapped;
@@ -264,7 +240,7 @@ int aio_write64(struct aiocb64 *aiocbp)
     /* Test to see if the I/O was queued successfully. */
     if (!IoOperationStatus) {
         errno = GetLastError();
-        debugs(81, 1, "aio_write: GetLastError=" << errno  );
+        debugs(81, DBG_IMPORTANT, "aio_write: GetLastError=" << errno  );
         return -1;
     }
 
@@ -274,19 +250,16 @@ int aio_write64(struct aiocb64 *aiocbp)
     return 0;
 }
 
-
 int aio_error(const struct aiocb * aiocbp)
 {
     return aiocbp->aio_sigevent.sigev_notify;
 }
 
-
 int aio_error64(const struct aiocb64 * aiocbp)
 {
     return aiocbp->aio_sigevent.sigev_notify;
 }
 
-
 int aio_open(const char *path, int mode)
 {
     HANDLE hndl;
@@ -311,15 +284,15 @@ int aio_open(const char *path, int mode)
     else
         dwCreationDisposition = (mode & O_CREAT) ? OPEN_ALWAYS : OPEN_EXISTING;
 
-    if ((hndl = CreateFile(path,	                /* file name               */
-                           dwDesiredAccess,	        /* access mode             */
-                           0,			            /* share mode              */
-                           NULL,			        /* SD                      */
-                           dwCreationDisposition,	/* how to create           */
-                           FILE_FLAG_OVERLAPPED,	/* file attributes         */
-                           NULL			            /* handle to template file */
+    if ((hndl = CreateFile(path,                    /* file name               */
+                           dwDesiredAccess,         /* access mode             */
+                           0,                       /* share mode              */
+                           NULL,                    /* SD                      */
+                           dwCreationDisposition,   /* how to create           */
+                           FILE_FLAG_OVERLAPPED,    /* file attributes         */
+                           NULL                     /* handle to template file */
                           )) != INVALID_HANDLE_VALUE) {
-        statCounter.syscalls.disk.opens++;
+        ++ statCounter.syscalls.disk.opens;
         fd = _open_osfhandle((long) hndl, 0);
         commSetCloseOnExec(fd);
         fd_open(fd, FD_FILE, path);
@@ -331,24 +304,22 @@ int aio_open(const char *path, int mode)
     return fd;
 }
 
-
 void aio_close(int fd)
 {
     CloseHandle((HANDLE)_get_osfhandle(fd));
     fd_close(fd);
-    statCounter.syscalls.disk.closes++;
+    ++ statCounter.syscalls.disk.closes;
 }
 
-
 ssize_t aio_return(struct aiocb * aiocbp)
 {
     return aiocbp->aio_sigevent.sigev_signo;
 }
 
-
 ssize_t aio_return64(struct aiocb64 * aiocbp)
 
 {
     return aiocbp->aio_sigevent.sigev_signo;
 }
 #endif /* _SQUID_WINDOWS_ */
+
